@@ -5,24 +5,8 @@ import sys
 import numpy as np
 import torch
 
-from cogdl.data import Dataset, Data
-from cogdl.utils import remove_self_loops, download_url, untar
-from . import register_dataset
-
-
-def coalesce(row, col, value=None):
-    num = col.shape[0] + 1
-    idx = torch.full((num,), -1, dtype=torch.float)
-    idx[1:] = row * num + col
-    mask = idx[1:] > idx[:-1]
-
-    if mask.all():
-        return row, col.value
-    row = row[mask]
-    col = col[mask]
-    if value is not None:
-        pass
-    return row, col, value
+from cogdl.data import Dataset, Graph
+from cogdl.utils import remove_self_loops, download_url, untar, coalesce, Accuracy, CrossEntropyLoss
 
 
 def parse_index_file(filename):
@@ -42,9 +26,9 @@ def edge_index_from_dict(graph_dict, num_nodes=None):
     row, col = [], []
     for key, value in graph_dict.items():
         row.append(np.repeat(key, len(value)))
-        col.append(value)
-    _row = np.concatenate(np.array(row))
-    _col = np.concatenate(np.array(col))
+        col.append(np.array(value))
+    _row = np.concatenate(row)
+    _col = np.concatenate(col)
     edge_index = np.stack([_row, _col], axis=0)
 
     row_dom = edge_index[:, _row > _col]
@@ -59,11 +43,11 @@ def edge_index_from_dict(graph_dict, num_nodes=None):
 
     edge_index = torch.tensor(edge_index, dtype=torch.long)
     # There may be duplicated edges and self loops in the datasets.
-    row, col, _ = coalesce(edge_index[0], edge_index[1])
-    edge_index = torch.stack([row, col], dim=0)
     edge_index, _ = remove_self_loops(edge_index)
     row = torch.cat([edge_index[0], edge_index[1]])
     col = torch.cat([edge_index[1], edge_index[0]])
+
+    row, col, _ = coalesce(row, col)
     edge_index = torch.stack([row, col])
     return edge_index
 
@@ -114,7 +98,7 @@ def read_planetoid_data(folder, prefix):
 
     edge_index = edge_index_from_dict(graph, num_nodes=y.size(0))
 
-    data = Data(x=x, edge_index=edge_index, y=y)
+    data = Graph(x=x, edge_index=edge_index, y=y)
     data.train_mask = train_mask
     data.val_mask = val_mask
     data.test_mask = test_mask
@@ -137,7 +121,6 @@ class Planetoid(Dataset):
         self.data = torch.load(self.processed_paths[0])
 
         self.split = split
-        assert self.split in ["public", "full"]
 
         self.raw_dir = osp.join(self.root, self.name, "raw")
         self.processed_dir = osp.join(self.root, self.name, "processed")
@@ -162,6 +145,11 @@ class Planetoid(Dataset):
         assert hasattr(self.data, "y")
         return int(torch.max(self.data.y)) + 1
 
+    @property
+    def num_nodes(self):
+        assert hasattr(self.data, "y")
+        return self.data.y.shape[0]
+
     def download(self):
         fname = "{}.zip".format(self.name.lower())
         download_url("{}{}.zip&dl=1".format(self.url, self.name.lower()), self.raw_dir, fname)
@@ -175,7 +163,16 @@ class Planetoid(Dataset):
         return self.data
 
     def __repr__(self):
-        return "{}()".format(self.name)
+        return "{}".format(self.name)
+
+    def __len__(self):
+        return 1
+
+    def get_evaluator(self):
+        return Accuracy()
+
+    def get_loss_fn(self):
+        return CrossEntropyLoss()
 
 
 def normalize_feature(data):
@@ -187,33 +184,30 @@ def normalize_feature(data):
     return data
 
 
-@register_dataset("cora")
 class CoraDataset(Planetoid):
-    def __init__(self):
+    def __init__(self, data_path="data"):
         dataset = "Cora"
-        path = osp.join("data", dataset)
+        path = osp.join(data_path, dataset)
         if not osp.exists(path):
             Planetoid(path, dataset)
         super(CoraDataset, self).__init__(path, dataset)
         self.data = normalize_feature(self.data)
 
 
-@register_dataset("citeseer")
 class CiteSeerDataset(Planetoid):
-    def __init__(self):
+    def __init__(self, data_path="data"):
         dataset = "CiteSeer"
-        path = osp.join("data", dataset)
+        path = osp.join(data_path, dataset)
         if not osp.exists(path):
             Planetoid(path, dataset)
         super(CiteSeerDataset, self).__init__(path, dataset)
         self.data = normalize_feature(self.data)
 
 
-@register_dataset("pubmed")
 class PubMedDataset(Planetoid):
-    def __init__(self):
+    def __init__(self, data_path="data"):
         dataset = "PubMed"
-        path = osp.join("data", dataset)
+        path = osp.join(data_path, dataset)
         if not osp.exists(path):
             Planetoid(path, dataset)
         super(PubMedDataset, self).__init__(path, dataset)
