@@ -10,10 +10,9 @@ import torch.nn as nn
 import torch.nn.functional as F
 from torch.nn.parameter import Parameter
 
-from .. import BaseModel, register_model
+from .. import BaseModel
 
 
-@register_model("gatne")
 class GATNE(BaseModel):
     r"""The GATNE model from the `"Representation Learning for Attributed Multiplex Heterogeneous Network"
     <https://dl.acm.org/doi/10.1145/3292500.3330964>`_ paper
@@ -23,7 +22,7 @@ class GATNE(BaseModel):
         walk_num (int) : The number of walks to sample for each node.
         window_size (int) : The actual context size which is considered in language model.
         worker (int) : The number of workers for word2vec.
-        epoch (int) : The number of training epochs.
+        epochs (int) : The number of training epochs.
         batch_size (int) : The size of each training batch.
         edge_dim (int) : Number of edge embedding dimensions.
         att_dim (int) : Number of attention dimensions.
@@ -45,8 +44,8 @@ class GATNE(BaseModel):
                             help='Window size of skip-gram model. Default is 5.')
         parser.add_argument('--worker', type=int, default=10,
                             help='Number of parallel workers. Default is 10.')
-        parser.add_argument('--epoch', type=int, default=20,
-                            help='Number of epoch. Default is 20.')
+        parser.add_argument('--epochs', type=int, default=20,
+                            help='Number of epochs. Default is 20.')
         parser.add_argument('--batch-size', type=int, default=256,
                             help='Number of batch_size. Default is 256.')
         parser.add_argument('--edge-dim', type=int, default=10,
@@ -69,7 +68,7 @@ class GATNE(BaseModel):
             args.walk_num,
             args.window_size,
             args.worker,
-            args.epoch,
+            args.epochs,
             args.batch_size,
             args.edge_dim,
             args.att_dim,
@@ -85,7 +84,7 @@ class GATNE(BaseModel):
         walk_num,
         window_size,
         worker,
-        epoch,
+        epochs,
         batch_size,
         edge_dim,
         att_dim,
@@ -99,17 +98,17 @@ class GATNE(BaseModel):
         self.walk_num = walk_num
         self.window_size = window_size
         self.worker = worker
-        self.epochs = epoch
+        self.epochs = epochs
         self.batch_size = batch_size
         self.embedding_u_size = edge_dim
         self.dim_att = att_dim
         self.num_sampled = negative_samples
         self.neighbor_samples = neighbor_samples
         self.schema = schema
-
         self.multiplicity = True
 
-    def train(self, network_data):
+    def forward(self, network_data):
+        device = "cpu" if not torch.cuda.is_available() else "cuda"
         all_walks = generate_walks(network_data, self.walk_num, self.walk_length, schema=self.schema)
         vocab, index2word = generate_vocab(all_walks)
         train_pairs = generate_pairs(all_walks, vocab)
@@ -153,8 +152,8 @@ class GATNE(BaseModel):
         model = GATNEModel(num_nodes, embedding_size, embedding_u_size, edge_type_count, dim_att)
         nsloss = NSLoss(num_nodes, num_sampled, embedding_size)
 
-        model.to(self.device)
-        nsloss.to(self.device)
+        model.to(device)
+        nsloss.to(device)
 
         optimizer = torch.optim.Adam([{"params": model.parameters()}, {"params": nsloss.parameters()}], lr=1e-4)
 
@@ -173,11 +172,11 @@ class GATNE(BaseModel):
             for i, data in enumerate(data_iter):
                 optimizer.zero_grad()
                 embs = model(
-                    data[0].to(self.device),
-                    data[2].to(self.device),
-                    data[3].to(self.device),
+                    data[0].to(device),
+                    data[2].to(device),
+                    data[3].to(device),
                 )
-                loss = nsloss(data[0].to(self.device), embs, data[1].to(self.device))
+                loss = nsloss(data[0].to(device), embs, data[1].to(device))
                 loss.backward()
                 optimizer.step()
 
@@ -194,9 +193,9 @@ class GATNE(BaseModel):
 
         final_model = dict(zip(edge_types, [dict() for _ in range(edge_type_count)]))
         for i in range(num_nodes):
-            train_inputs = torch.tensor([i for _ in range(edge_type_count)]).to(self.device)
-            train_types = torch.tensor(list(range(edge_type_count))).to(self.device)
-            node_neigh = torch.tensor([neighbors[i] for _ in range(edge_type_count)]).to(self.device)
+            train_inputs = torch.tensor([i for _ in range(edge_type_count)]).to(device)
+            train_types = torch.tensor(list(range(edge_type_count))).to(device)
+            node_neigh = torch.tensor([neighbors[i] for _ in range(edge_type_count)]).to(device)
             node_emb = model(train_inputs, train_types, node_neigh)
             for j in range(edge_type_count):
                 final_model[edge_types[j]][index2word[i]] = node_emb[j].cpu().detach().numpy()
