@@ -2,10 +2,9 @@ import numpy as np
 import networkx as nx
 from gensim.models import Word2Vec, KeyedVectors
 import random
-from .. import BaseModel, register_model
+from .. import BaseModel
 
 
-@register_model("metapath2vec")
 class Metapath2vec(BaseModel):
     r"""The Metapath2vec model from the `"metapath2vec: Scalable Representation
     Learning for Heterogeneous Networks"
@@ -64,9 +63,12 @@ class Metapath2vec(BaseModel):
         self.schema = schema
         self.node_type = None
 
-    def train(self, G, node_type):
+    def forward(self, data):
+        G = nx.DiGraph()
+        row, col = data.edge_index
+        G.add_edges_from(list(zip(row.numpy(), col.numpy())))
         self.G = G
-        self.node_type = [str(a) for a in node_type]
+        self.node_type = [str(a) for a in data.pos.tolist()]
         walks = self._simulate_walks(self.walk_length, self.walk_num, self.schema)
         walks = [[str(node) for node in walk] for walk in walks]
         model = Word2Vec(
@@ -110,7 +112,6 @@ class Metapath2vec(BaseModel):
         nodes = list(G.nodes())
         if schema != "No":
             schema_list = schema.split(",")
-        print("node number:", len(nodes))
         for walk_iter in range(num_walks):
             random.shuffle(nodes)
             print(str(walk_iter + 1), "/", str(num_walks))
