@@ -1,28 +1,19 @@
 import torch
+
+# import torch.fft
 import torch.nn as nn
 import torch.nn.functional as F
 
+from cogdl.data import Graph
 from cogdl.utils import row_normalization
-from cogdl.layers.link_prediction_module import GNNLinkPredict, sampling_edge_uniform, cal_mrr
-from .. import BaseModel, register_model
+from cogdl.utils.link_prediction_utils import GNNLinkPredict, sampling_edge_uniform, cal_mrr
+from .. import BaseModel
 
 
-def com_mult(a, b):
-    """Borrowed from https://github.com/malllabiisc/CompGCN"""
-    r1, i1 = a[..., 0], a[..., 1]
-    r2, i2 = b[..., 0], b[..., 1]
-    return torch.stack([r1 * r2 - i1 * i2, r1 * i2 + i1 * r2], dim=-1)
-
-
-def conj(a):
-    """Borrowed from https://github.com/malllabiisc/CompGCN"""
-    a[..., 1] = -a[..., 1]
-    return a
-
-
-def ccorr(a, b):
-    """Borrowed from https://github.com/malllabiisc/CompGCN"""
-    return torch.irfft(com_mult(conj(torch.rfft(a, 1)), torch.rfft(b, 1)), 1, signal_sizes=(a.shape[-1],))
+# def ccorr(a, b):
+#     return torch.fft.irfft(
+#         torch.multiply(torch.fft.rfft(a, dim=1).conj(), torch.fft.rfft(b, dim=1)), n=a.shape[-1], dim=1
+#     )
 
 
 class BasesRelEmbLayer(nn.Module):
@@ -100,24 +91,28 @@ class CompGCNLayer(nn.Module):
         nn.init.xavier_normal_(weight.data)
         return weight
 
-    def forward(self, x, edge_index, edge_type, rel_embed=None):
+    def forward(self, graph, x, rel_embed):
         device = x.device
         if self.use_bases:
             rel_embed = self.basis_weight()
+        edge_index = graph.edge_index
+        edge_type = graph.edge_attr
         rel_embed = torch.cat((rel_embed, self.loop_rel), dim=0)
-        num_edges = edge_index.shape[1] // 2
+        num_edges = edge_index[0].shape[0] // 2
         num_entities = x.shape[0]
 
-        index, rev_index = edge_index[:, :num_edges], edge_index[:, num_edges:]
+        row, col = edge_index
+        i_row, i_col = row[:num_edges], col[:num_edges]
+        rev_row, rev_col = row[num_edges:], col[num_edges:]
         loop_index = torch.stack((torch.arange(num_entities), torch.arange(num_entities))).to(device)
         types, rev_types = edge_type[:num_edges], edge_type[num_edges:]
         loop_types = torch.full((num_entities,), rel_embed.shape[0] - 1, dtype=torch.long).to(device)
 
-        in_norm = row_normalization(num_entities, index)
-        rev_norm = row_normalization(num_entities, rev_index)
+        in_norm = row_normalization(num_entities, i_row, i_col)
+        rev_norm = row_normalization(num_entities, rev_row, rev_col)
 
-        emb = self.message_passing(x, rel_embed, index, types, "in", in_norm)
-        rev_emb = self.message_passing(x, rel_embed, rev_index, rev_types, "out", rev_norm)
+        emb = self.message_passing(x, rel_embed, (i_row, i_col), types, "in", in_norm)
+        rev_emb = self.message_passing(x, rel_embed, (rev_row, rev_col), rev_types, "out", rev_norm)
         loop_emb = self.message_passing(x, rel_embed, loop_index, loop_types, "loop")
 
         out = 1 / 3 * (emb + rev_emb + loop_emb)
@@ -145,23 +140,23 @@ class CompGCNLayer(nn.Module):
         return F.dropout(embed, p=self.dropout, training=self.training)
 
     def rel_transform(self, ent_embed, rel_embed):
-        if self.opn == "corr":
-            trans_embed = ccorr(ent_embed, rel_embed)
-        elif self.opn == "sub":
+        # if self.opn == "corr":
+        #     trans_embed = ccorr(ent_embed, rel_embed)
+        if self.opn == "sub":
             trans_embed = ent_embed - rel_embed
         elif self.opn == "mult":
             trans_embed = ent_embed * rel_embed
         else:
-            raise NotImplementedError
+            raise NotImplementedError(f"{self.opn}")
         return trans_embed
 
 
 class CompGCN(nn.Module):
     def __init__(
-        self, num_entities, num_rels, num_bases, in_feats, hidden_size, out_feats, layers, dropout, activation
+        self, num_entities, num_rels, num_bases, in_feats, hidden_size, out_feats, layers, dropout, activation, opn
     ):
         super(CompGCN, self).__init__()
-        self.opn = "corr"
+        self.opn = opn
         self.num_rels = num_rels
         self.num_entities = num_entities
         if num_bases is not None and num_bases > 0:
@@ -205,15 +200,14 @@ class CompGCN(nn.Module):
                 )
             )
 
-    def forward(self, x, edge_index, edge_types):
+    def forward(self, graph, x):
         rel_embed = self.init_rel.weight
         node_embed = x
         for layer in self.convs:
-            node_embed, rel_embed = layer(node_embed, edge_index, edge_types, rel_embed)
+            node_embed, rel_embed = layer(graph, node_embed, rel_embed)
         return node_embed, rel_embed
 
 
-@register_model("compgcn")
 class LinkPredictCompGCN(GNNLinkPredict, BaseModel):
     @staticmethod
     def add_args(parser):
@@ -224,8 +218,8 @@ class LinkPredictCompGCN(GNNLinkPredict, BaseModel):
         parser.add_argument("--num-bases", type=int, default=10)
         parser.add_argument("--num-layers", type=int, default=1)
         parser.add_argument("--sampling-rate", type=float, default=0.01)
-        parser.add_argument("--score-func", type=str, default="conve")
         parser.add_argument("--lbl_smooth", type=float, default=0.1)
+        parser.add_argument("--opn", type=str, default="sub")
         # fmt: on
 
     @classmethod
@@ -236,11 +230,11 @@ class LinkPredictCompGCN(GNNLinkPredict, BaseModel):
             hidden_size=args.hidden_size,
             num_bases=args.num_bases,
             sampling_rate=args.sampling_rate,
-            score_func=args.score_func,
             penalty=args.penalty,
             layers=args.num_layers,
             dropout=args.dropout,
             lbl_smooth=args.lbl_smooth,
+            opn=args.opn,
         )
 
     def __init__(
@@ -251,19 +245,26 @@ class LinkPredictCompGCN(GNNLinkPredict, BaseModel):
         num_bases=0,
         layers=1,
         sampling_rate=0.01,
-        score_func="conve",
         penalty=0.001,
         dropout=0.0,
         lbl_smooth=0.1,
+        opn="sub",
     ):
         BaseModel.__init__(self)
-        GNNLinkPredict.__init__(self, score_func, hidden_size)
+        GNNLinkPredict.__init__(self)
         activation = F.tanh
         self.model = CompGCN(
-            num_entities, num_rels, num_bases, hidden_size // 2, hidden_size, hidden_size, layers, dropout, activation
+            num_entities,
+            num_rels,
+            num_bases,
+            hidden_size // 2,
+            hidden_size,
+            hidden_size,
+            layers,
+            dropout,
+            activation,
+            opn,
         )
-        # self.emb = nn.Parameter(torch.Tensor(num_entities, hidden_size))
-        # nn.init.xavier_uniform_(self.emb)
         self.emb = nn.Embedding(num_entities, hidden_size // 2)
         self.sampling_rate = sampling_rate
         self.penalty = penalty
@@ -273,32 +274,33 @@ class LinkPredictCompGCN(GNNLinkPredict, BaseModel):
         self.lbl_smooth = lbl_smooth
 
     def add_reverse_edges(self, edge_index, edge_types):
-        edge_index_rev = torch.stack([edge_index[1], edge_index[0]])
-        edge_index = torch.cat([edge_index, edge_index_rev], dim=1)
+        row, col = edge_index
+        _row = torch.cat([row, col])
+        _col = torch.cat([col, row])
+        edge_index = torch.cat((_row, _col), dim=1)
         edge_types_rev = edge_types + self.num_rels
         edge_types = torch.cat([edge_types, edge_types_rev])
         return edge_index, edge_types
 
-    def forward(self, edge_index, edge_types):
+    def forward(self, graph):
+        edge_index = torch.stack(graph.edge_index)
         # edge_index, edge_types = self.add_reverse_edges(edge_index, edge_types)
         reindexed_node, reindexed_edge_index = torch.unique(edge_index, return_inverse=True, sorted=True)
         self.cache_index = reindexed_node
         node_embed = self.emb(reindexed_node)
-        node_embed, rel_embed = self.model(node_embed, reindexed_edge_index, edge_types)
+        with graph.local_graph():
+            graph.edge_index = reindexed_edge_index
+            node_embed, rel_embed = self.model(graph, node_embed)
         return node_embed, rel_embed
 
-    def loss(self, data, split="train"):
-        if split == "train":
-            mask = data.train_mask
-        elif split == "val":
-            mask = data.val_mask
-        else:
-            mask = data.test_mask
-        edge_index, edge_types = data.edge_index[:, mask], data.edge_attr[mask]
+    def loss(self, data: Graph, scoring):
+        row, col = data.edge_index
+        edge_types = data.edge_attr
+        edge_index = torch.stack([row, col])
 
         self.get_edge_set(edge_index, edge_types)
         batch_edges, batch_attr, samples, rels, labels = sampling_edge_uniform(
-            edge_index,
+            (row, col),
             edge_types,
             self.edge_set,
             self.sampling_rate,
@@ -306,34 +308,22 @@ class LinkPredictCompGCN(GNNLinkPredict, BaseModel):
             label_smoothing=self.lbl_smooth,
             num_entities=self.num_entities,
         )
-        node_embed, rel_embed = self.forward(batch_edges, batch_attr)
+        with data.local_graph():
+            data.edge_index = batch_edges
+            data.edge_attr = batch_attr
+            node_embed, rel_embed = self.forward(data)
 
         sampled_nodes, reindexed_edges = torch.unique(samples, sorted=True, return_inverse=True)
         assert (self.cache_index == sampled_nodes).any()
-        loss_n = self._loss(node_embed[reindexed_edges[0]], node_embed[reindexed_edges[1]], rel_embed[rels], labels)
+        loss_n = self._loss(
+            node_embed[reindexed_edges[0]], node_embed[reindexed_edges[1]], rel_embed[rels], labels, scoring
+        )
         loss_r = self.penalty * self._regularization([self.emb(sampled_nodes), rel_embed])
         return loss_n + loss_r
 
-        # Full graph as input
-        # sampled_nodes = torch.unique(samples)
-        # node_embed, rel_embed = self.forward(edge_index, edge_types)
-        # loss_n = self._loss(node_embed[samples[0]], node_embed[samples[1]], rel_embed[rels], labels)
-        # loss_r = self.penalty * self._regularization([self.emb(sampled_nodes), rel_embed])
-        # return loss_n + loss_r
-
-    def predict(self, edge_index, edge_types):
-        indices = torch.arange(0, self.num_entities).to(edge_index.device)
+    def predict(self, graph):
+        device = next(self.parameters()).device
+        indices = torch.arange(0, self.num_entities).to(device)
         x = self.emb(indices)
-        # edge_index, edge_types = self.add_reverse_edges(edge_index, edge_types)
-        node_embed, rel_embed = self.model(x, edge_index, edge_types)
-        mrr, hits = cal_mrr(
-            node_embed,
-            rel_embed,
-            edge_index,
-            edge_types,
-            scoring=self.scoring,
-            protocol="raw",
-            batch_size=500,
-            hits=[1, 3, 10],
-        )
-        return mrr, hits
+        node_embed, rel_embed = self.model(graph, x)
+        return node_embed, rel_embed
