@@ -1,12 +1,12 @@
-from typing import Optional, Any, List
+from typing import List
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
-from .. import register_model, BaseModel
-from cogdl.models.nn.gcn import GraphConvolution
-from cogdl.utils import get_activation, filter_adj, add_remaining_self_loops, symmetric_normalization
-from cogdl.trainers.self_supervised_trainer import SelfSupervisedTrainer
+from .. import BaseModel
+from cogdl.layers import GCNLayer
+from cogdl.utils import get_activation
+from cogdl.data import Graph
 
 
 class GraceEncoder(nn.Module):
@@ -19,18 +19,17 @@ class GraceEncoder(nn.Module):
     ):
         super(GraceEncoder, self).__init__()
         shapes = [in_feats] + [2 * out_feats] * (num_layers - 1) + [out_feats]
-        self.layers = nn.ModuleList([GraphConvolution(shapes[i], shapes[i + 1]) for i in range(num_layers)])
+        self.layers = nn.ModuleList([GCNLayer(shapes[i], shapes[i + 1]) for i in range(num_layers)])
         self.activation = get_activation(activation)
 
-    def forward(self, x: torch.Tensor, edge_index: torch.Tensor, edge_weight: torch.Tensor):
+    def forward(self, graph: Graph, x: torch.Tensor):
         h = x
         for layer in self.layers:
-            h = layer(h, edge_index, edge_weight)
+            h = layer(graph, h)
             h = self.activation(h)
         return h
 
 
-@register_model("grace")
 class GRACE(BaseModel):
     @staticmethod
     def add_args(parser):
@@ -83,28 +82,30 @@ class GRACE(BaseModel):
         )
         self.encoder = GraceEncoder(in_feats, hidden_size, num_layers, activation)
 
+    def augment(self, graph):
+        pass
+
     def forward(
         self,
-        x: torch.Tensor,
-        edge_index: torch.Tensor,
-        edge_weight: Optional[torch.Tensor] = None,
+        graph: Graph,
+        x: torch.Tensor = None,
     ):
-        num_nodes = x.shape[0]
-        edge_index, edge_weight = add_remaining_self_loops(edge_index, edge_weight)
-        edge_weight = symmetric_normalization(num_nodes, edge_index)
-        return self.encoder(x, edge_index, edge_weight)
+        if x is None:
+            x = graph.x
+        graph.sym_norm()
+        return self.encoder(graph, x)
 
     def prop(
         self,
+        graph: Graph,
         x: torch.Tensor,
-        edge_index: torch.Tensor,
-        edge_weight: Optional[torch.Tensor] = None,
         drop_feature_rate: float = 0.0,
         drop_edge_rate: float = 0.0,
     ):
         x = self.drop_feature(x, drop_feature_rate)
-        edge_index, edge_weight = self.drop_adj(edge_index, edge_weight, drop_edge_rate)
-        return self.forward(x, edge_index, edge_weight)
+        with graph.local_graph():
+            graph = self.drop_adj(graph, drop_edge_rate)
+            return self.forward(graph, x)
 
     def contrastive_loss(self, z1: torch.Tensor, z2: torch.Tensor):
         z1 = F.normalize(z1, p=2, dim=-1)
@@ -138,41 +139,32 @@ class GRACE(BaseModel):
             losses.append(_loss)
         return sum(losses) / len(losses)
 
-    def node_classification_loss(self, data):
-        z1 = self.prop(data.x, data.edge_index, data.edge_attr, self.drop_feature_rates[0], self.drop_edge_rates[0])
-        z2 = self.prop(data.x, data.edge_index, data.edge_attr, self.drop_feature_rates[1], self.drop_edge_rates[1])
-
-        z1 = self.project_head(z1)
-        z2 = self.project_head(z2)
-
-        if self.batch_size > 0:
-            return 0.5 * (self.batched_loss(z1, z2, self.batch_size) + self.batched_loss(z2, z1, self.batch_size))
-        else:
-            return 0.5 * (self.contrastive_loss(z1, z2) + self.contrastive_loss(z2, z1))
-
     def embed(self, data):
-        pred = self.forward(data.x, data.edge_index, data.edge_attr)
+        pred = self.forward(data, data.x)
         return pred
 
-    def drop_adj(self, edge_index: torch.Tensor, edge_weight: Optional[torch.Tensor] = None, drop_rate: float = 0.5):
+    def drop_adj(self, graph: Graph, drop_rate: float = 0.5):
         if drop_rate < 0.0 or drop_rate > 1.0:
             raise ValueError("Dropout probability has to be between 0 and 1, " "but got {}".format(drop_rate))
         if not self.training:
-            return edge_index, edge_weight
+            return graph
 
-        mask = edge_index.new_full((edge_index.size(1),), 1 - drop_rate, dtype=torch.float)
+        num_edges = graph.num_edges
+        mask = torch.full((num_edges,), 1 - drop_rate, dtype=torch.float)
         mask = torch.bernoulli(mask).to(torch.bool)
-        edge_index, edge_weight = filter_adj(edge_index[0], edge_index[1], edge_weight, mask)
-        return edge_index, edge_weight
+        row, col = graph.edge_index
+        row = row[mask]
+        col = col[mask]
+        edge_weight = graph.edge_weight[mask]
+        graph.edge_index = (row, col)
+        graph.edge_weight = edge_weight
+        return graph
 
     def drop_feature(self, x: torch.Tensor, droprate: float):
         n = x.shape[1]
         drop_rates = torch.ones(n) * droprate
         if self.training:
-            masks = torch.bernoulli(1.0 - drop_rates).unsqueeze(0).expand_as(x)
-            x = masks.to(x.device) * x
+            masks = torch.bernoulli(1.0 - drop_rates).view(1, -1).expand_as(x)
+            masks = masks.to(x.device)
+            x = masks * x
         return x
-
-    @staticmethod
-    def get_trainer(task, args):
-        return SelfSupervisedTrainer
